@@ -52,8 +52,8 @@
 #'     The coefficient matrix, if the system of differential equations can be
 #'     represented by one.
 #'   }
-#'   \item{ll}{
-#'     The likelihood function, taking the parameter vector as the first argument.
+#'   \item{cf}{
+#'     If generated, the compiled function as returned by cfunction.
 #'   }
 #' @note The IORE submodel is not well tested for metabolites. When using this
 #'   model for metabolites, you may want to read the second note in the help
@@ -70,7 +70,7 @@
 #' @examples
 #'
 #' # Specify the SFO model (this is not needed any more, as we can now mkinfit("SFO", ...)
-#' SFO <- mkinmod(parent = list(type = "SFO"))
+#' SFO <- mkinmod(parent = mkinsub("SFO"))
 #'
 #' # One parent compound, one metabolite, both single first order
 #' SFO_SFO <- mkinmod(
@@ -101,7 +101,7 @@
 #' }
 #'
 #' @export mkinmod
-mkinmod <- function(..., use_of_ff = "min", speclist = NULL, quiet = FALSE, verbose = FALSE)
+mkinmod <- function(..., use_of_ff = "max", speclist = NULL, quiet = FALSE, verbose = FALSE)
 {
   if (is.null(speclist)) spec <- list(...)
   else spec <- speclist
@@ -117,13 +117,6 @@ mkinmod <- function(..., use_of_ff = "min", speclist = NULL, quiet = FALSE, verb
   if (!use_of_ff %in% c("min", "max"))
     stop("The use of formation fractions 'use_of_ff' can only be 'min' or 'max'")
 
-  # The returned model will be a list of character vectors, containing {{{
-  # differential equations (if supported), parameter names and a mapping from
-  # model variables to observed variables. If possible, a matrix representation
-  # of the differential equations is included
-  # Compiling the functions from the C code generated below only works if the
-  # implicit assumption about differential equations specified below
-  # is satisfied
   parms <- vector()
   # }}}
 
@@ -182,7 +175,7 @@ mkinmod <- function(..., use_of_ff = "min", speclist = NULL, quiet = FALSE, verb
                 "are only supported with formation fractions (use_of_ff = 'max')")
         }
         if(spec[[varname]]$sink) {
-          # If sink is required, add first-order/IORE sink term
+          # If sink is requested, add first-order/IORE sink term
           k_compound_sink <- paste("k", box_1, "sink", sep = "_")
           if(spec[[varname]]$type == "IORE") {
             k_compound_sink <- paste("k__iore", box_1, "sink", sep = "_")
@@ -197,7 +190,7 @@ mkinmod <- function(..., use_of_ff = "min", speclist = NULL, quiet = FALSE, verb
         } else { # otherwise no decline term needed here
           decline_term = "0"
         }
-      } else {
+      } else { # Maximum use of formation fractions
         k_compound <- paste("k", box_1, sep = "_")
         if(spec[[varname]]$type == "IORE") {
           k_compound <- paste("k__iore", box_1, sep = "_")
@@ -236,26 +229,13 @@ mkinmod <- function(..., use_of_ff = "min", speclist = NULL, quiet = FALSE, verb
     diffs[[box_1]] <- paste(diffs[[box_1]], "-", decline_term)#}}}
     if(spec[[varname]]$type == "SFORB") { # {{{ Add SFORB reversible binding terms
       box_2 = map[[varname]][[2]]
-      if (use_of_ff == "min") { # Minimum use of formation fractions
-        k_free_bound <- paste("k", varname, "free", "bound", sep = "_")
-        k_bound_free <- paste("k", varname, "bound", "free", sep = "_")
-        parms <- c(parms, k_free_bound, k_bound_free)
-        reversible_binding_term_1 <- paste("-", k_free_bound, "*", box_1, "+",
-          k_bound_free, "*", box_2)
-        reversible_binding_term_2 <- paste("+", k_free_bound, "*", box_1, "-",
-          k_bound_free, "*", box_2)
-      } else { # Use formation fractions also for the free compartment
-        stop("The maximum use of formation fractions is not supported for SFORB models")
-        # The problems were: Calculation of dissipation times did not work in this case
-        # and the coefficient matrix is not generated correctly by the code present
-        # in this file in this case
-        #f_free_bound <- paste("f", varname, "free", "bound", sep = "_")
-        #k_bound_free <- paste("k", varname, "bound", "free", sep = "_")
-        #parms <- c(parms, f_free_bound, k_bound_free)
-        #reversible_binding_term_1 <- paste("+", k_bound_free, "*", box_2)
-        #reversible_binding_term_2 <- paste("+", f_free_bound, "*", k_compound, "*", box_1, "-",
-        #  k_bound_free, "*", box_2)
-      }
+      k_free_bound <- paste("k", varname, "free", "bound", sep = "_")
+      k_bound_free <- paste("k", varname, "bound", "free", sep = "_")
+      parms <- c(parms, k_free_bound, k_bound_free)
+      reversible_binding_term_1 <- paste("-", k_free_bound, "*", box_1, "+",
+        k_bound_free, "*", box_2)
+      reversible_binding_term_2 <- paste("+", k_free_bound, "*", box_1, "-",
+        k_bound_free, "*", box_2)
       diffs[[box_1]] <- paste(diffs[[box_1]], reversible_binding_term_1)
       diffs[[box_2]] <- paste(diffs[[box_2]], reversible_binding_term_2)
     } #}}}
@@ -286,7 +266,7 @@ mkinmod <- function(..., use_of_ff = "min", speclist = NULL, quiet = FALSE, verb
             k_from_to, "*", origin_box)
         } else {
           # Do not introduce a formation fraction if this is the only target
-          if (spec[[origin_box]]$sink == FALSE && n_targets == 1) {
+          if (spec[[varname]]$sink == FALSE && n_targets == 1) {
             diffs[[target_box]] <- paste(diffs[[target_box]], "+",
                                          decline_term)
           } else {
@@ -302,7 +282,7 @@ mkinmod <- function(..., use_of_ff = "min", speclist = NULL, quiet = FALSE, verb
 
   model <- list(diffs = diffs, parms = parms, map = map, spec = spec, use_of_ff = use_of_ff)
 
-  # Create coefficient matrix if appropriate#{{{
+  # Create coefficient matrix if possible #{{{
   if (mat) {
     boxes <- names(diffs)
     n <- length(boxes)
@@ -321,12 +301,12 @@ mkinmod <- function(..., use_of_ff = "min", speclist = NULL, quiet = FALSE, verb
 
           } else {          # off-diagonal elements
             k.candidate = paste("k", from, to, sep = "_")
-      if (sub("_free$", "", from) == sub("_bound$", "", to)) {
+            if (sub("_free$", "", from) == sub("_bound$", "", to)) {
               k.candidate = paste("k", sub("_free$", "_free_bound", from), sep = "_")
-      }
-      if (sub("_bound$", "", from) == sub("_free$", "", to)) {
+            }
+            if (sub("_bound$", "", from) == sub("_free$", "", to)) {
               k.candidate = paste("k", sub("_bound$", "_bound_free", from), sep = "_")
-      }
+            }
             k.effective = intersect(model$parms, k.candidate)
             m[to, from] = ifelse(length(k.effective) > 0,
                 k.effective, "0")
@@ -350,7 +330,8 @@ mkinmod <- function(..., use_of_ff = "min", speclist = NULL, quiet = FALSE, verb
           } else {          # off-diagonal elements
             f.candidate = paste("f", from, "to", to, sep = "_")
             k.candidate = paste("k", from, to, sep = "_")
-            # SFORB with maximum use of formation fractions not implemented, see above
+            k.candidate = sub("free.*bound", "free_bound", k.candidate)
+            k.candidate = sub("bound.*free", "bound_free", k.candidate)
             m[to, from] = ifelse(f.candidate %in% model$parms,
               paste(f.candidate, " * k_", from, sep = ""),
               ifelse(k.candidate %in% model$parms, k.candidate, "0"))
@@ -365,7 +346,7 @@ mkinmod <- function(..., use_of_ff = "min", speclist = NULL, quiet = FALSE, verb
     model$coefmat <- m
   }#}}}
 
-  # Try to create a function compiled from C code there is more than one observed variable {{{
+  # Try to create a function compiled from C code if there is more than one observed variable {{{
   # and a compiler is available
   if (length(obs_vars) > 1 & has_compiler()) {
 
@@ -440,6 +421,9 @@ mkinmod <- function(..., use_of_ff = "min", speclist = NULL, quiet = FALSE, verb
   }
   # }}}
 
+  # Attach a degradation function if an analytical solution is available
+  model$deg_func <- create_deg_func(spec, use_of_ff)
+
   class(model) <- "mkinmod"
   return(model)
 }
