@@ -69,6 +69,7 @@ if(getRversion() >= '2.15.1') utils::globalVariables(c("type", "variable", "obse
 #' \dontrun{
 #' SFO_SFO <- mkinmod(parent = mkinsub("SFO", "m1", full = "Parent"),
 #'                    m1 = mkinsub("SFO", full = "Metabolite M1" ))
+#' fit <- mkinfit(SFO_SFO, FOCUS_2006_D, quiet = TRUE)
 #' fit <- mkinfit(SFO_SFO, FOCUS_2006_D, quiet = TRUE, error_model = "tc")
 #' plot(fit)
 #' plot_res(fit)
@@ -147,6 +148,7 @@ plot.mkinfit <- function(x, fit = x,
              solution_type = solution_type, atol = fit$atol, rtol = fit$rtol,
              use_compiled = FALSE)
   }
+  out <- as.data.frame(out)
 
   names(col_obs) <- names(pch_obs) <- names(lty_obs) <- obs_vars
 
